"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
//================================================================ 
/**
 * @packageDocumentation
 * @module std.ranges
 */
//================================================================
var base = __importStar(require("../../algorithm/merge"));
var factory_1 = require("../../iterator/factory");
var global_1 = require("../../iterator/global");
var comparators_1 = require("../../functional/comparators");
/* ---------------------------------------------------------
    MERGE
--------------------------------------------------------- */
/**
 * Merge two sorted ranges.
 *
 * @param range1 The 1st iterable ranged container.
 * @param range2 The 2nd iterable ranged container.
 * @param output Output iterator of the first position.
 * @param comp A binary function predicates *x* element would be placed before *y*. When returns `true`, then *x* precedes *y*. Default is {@link less}.
 *
 * @return Output Iterator of the last position by advancing.
 */
function merge(range1, range2, output, comp) {
    if (comp === void 0) { comp = comparators_1.less; }
    return base.merge(factory_1.begin(range1), factory_1.end(range1), factory_1.begin(range2), factory_1.end(range2), output, comp);
}
exports.merge = merge;
/**
 * Merge two sorted & consecutive ranges.
 *
 * @param range An iterable ranged container.
 * @param middle Bidirectional iterator of the initial position of the 2nd range.
 * @param comp A binary function predicates *x* element would be placed before *y*. When returns `true`, then *x* precedes *y*. Default is {@link less}.
 */
function inplace_merge(range, middle, comp) {
    if (comp === void 0) { comp = comparators_1.less; }
    return base.inplace_merge(factory_1.begin(range), middle, factory_1.end(range), comp);
}
exports.inplace_merge = inplace_merge;
/* ---------------------------------------------------------
    SET OPERATIONS
--------------------------------------------------------- */
/**
 * Test whether two sorted ranges are in inclusion relationship.
 *
 * @param range1 The 1st iterable ranged container.
 * @param range2 The 2nd iterable ranged container.
 * @param comp A binary function predicates *x* element would be placed before *y*. When returns `true`, then *x* precedes *y*. Default is {@link less}.
 *
 * @return Whether [first, last1) includes [first2, last2).
 */
function includes(range1, range2, comp) {
    if (comp === void 0) { comp = comparators_1.less; }
    if (global_1.size(range1) < global_1.size(range2))
        return false;
    else
        return base.includes(factory_1.begin(range1), factory_1.end(range1), factory_1.begin(range2), factory_1.end(range2), comp);
}
exports.includes = includes;
/**
 * Combine two sorted ranges to union relationship.
 *
 * @param range1 The 1st iterable ranged container.
 * @param range2 The 2nd iterable ranged container.
 * @param output Output iterator of the first position.
 * @param comp A binary function predicates *x* element would be placed before *y*. When returns `true`, then *x* precedes *y*. Default is {@link less}.
 *
 * @return Output Iterator of the last position by advancing.
 */
function set_union(range1, range2, output, comp) {
    if (comp === void 0) { comp = comparators_1.less; }
    return base.set_union(factory_1.begin(range1), factory_1.end(range1), factory_1.begin(range2), factory_1.end(range2), output, comp);
}
exports.set_union = set_union;
/**
 * Combine two sorted ranges to intersection relationship.
 *
 * @param range1 The 1st iterable ranged container.
 * @param range2 The 2nd iterable ranged container.
 * @param output Output iterator of the first position.
 * @param comp A binary function predicates *x* element would be placed before *y*. When returns `true`, then *x* precedes *y*. Default is {@link less}.
 *
 * @return Output Iterator of the last position by advancing.
 */
function set_intersection(range1, range2, output, comp) {
    if (comp === void 0) { comp = comparators_1.less; }
    return base.set_intersection(factory_1.begin(range1), factory_1.end(range1), factory_1.begin(range2), factory_1.end(range2), output, comp);
}
exports.set_intersection = set_intersection;
/**
 * Combine two sorted ranges to difference relationship.
 *
 * @param range1 The 1st iterable ranged container.
 * @param range2 The 2nd iterable ranged container.
 * @param output Output iterator of the first position.
 * @param comp A binary function predicates *x* element would be placed before *y*. When returns `true`, then *x* precedes *y*. Default is {@link less}.
 *
 * @return Output Iterator of the last position by advancing.
 */
function set_difference(range1, range2, output, comp) {
    if (comp === void 0) { comp = comparators_1.less; }
    return base.set_difference(factory_1.begin(range1), factory_1.end(range1), factory_1.begin(range2), factory_1.end(range2), output, comp);
}
exports.set_difference = set_difference;
/**
 * Combine two sorted ranges to symmetric difference relationship.
 *
 * @param range1 The 1st iterable ranged container.
 * @param range2 The 2nd iterable ranged container.
 * @param output Output iterator of the first position.
 * @param comp A binary function predicates *x* element would be placed before *y*. When returns `true`, then *x* precedes *y*. Default is {@link less}.
 *
 * @return Output Iterator of the last position by advancing.
 */
function set_symmetric_difference(range1, range2, output, comp) {
    if (comp === void 0) { comp = comparators_1.less; }
    return base.set_symmetric_difference(factory_1.begin(range1), factory_1.end(range1), factory_1.begin(range2), factory_1.end(range2), output, comp);
}
exports.set_symmetric_difference = set_symmetric_difference;
//# sourceMappingURL=merge.js.map